@@ -2,13 +2,12 @@
 # IMPORTS
 #######################################
 
-from Celeratas.helper.tokens import *
-from Celeratas.helper.errors import IndexingError, NamingError, AttrError, RTError, TypingError
+import Celeratas.helper.tokens as toks
+from Celeratas.helper.errors import (AttrError, IndexingError, NamingError,
+                                     RecursingError, RTError, TypingError)
 
-from .values import *
-from .constants import *
 from .RTResult import RTResult
-
+from .values import Bool, Dict, Function, List, Number, Numeral, String
 
 #######################################
 # INTERPRETER
@@ -16,6 +15,9 @@ from .RTResult import RTResult
 
 
 class Interpreter:
+    def __init__(self, recursion_depth):
+        self.recursion_depth = recursion_depth
+
     def visit(self, node, context):
         method_name = f'visit_{type(node).__name__}'
         method = getattr(self, method_name, self.no_visit_method)
@@ -28,13 +30,13 @@ class Interpreter:
 
     def visit_NumberNode(self, node, context):
         return RTResult().success(
-            Number(node.tok.value).set_context(
+            Number(node.value).set_context(
                 context).set_pos(node.pos_start, node.pos_end)
         )
 
     def visit_NumeralNode(self, node, context):
         return RTResult().success(
-            Numeral(node.tok.value).set_context(
+            Numeral(node.value).set_context(
                 context).set_pos(node.pos_start, node.pos_end)
         )
 
@@ -60,7 +62,7 @@ class Interpreter:
 
     def visit_BoolNode(self, node, context):
         return RTResult().success(
-            Bool(node.tok.value == "Verus").set_context(
+            Bool(node.value == "Verus").set_context(
                 context).set_pos(node.pos_start, node.pos_end)
         )
 
@@ -105,20 +107,28 @@ class Interpreter:
 
     def visit_VarAccessNode(self, node, context):
         res = RTResult()
-        var_name = node.var_name_tok.value
-        value = context.symbol_table.get(var_name)
+        var_name_to_get = node.var_name_to_get
+        value = context.symbol_table.get(var_name_to_get)
         idxes_to_get = node.idxes_to_get
-        attr_to_get = node.attr_to_get
+        attrs_to_get = node.attrs_to_get
 
         if not value:
             return res.failure(NamingError(
                 node.pos_start, node.pos_end,
-                f"'{var_name}' is not defined",
+                f"'{var_name_to_get}' is not defined",
                 context
             ))
 
         value = value.copy().set_pos(node.pos_start, node.pos_end).set_context(context)
 
+        # Check if it is not here before loop because it won't check a value that is from an index
+        if idxes_to_get and not isinstance(value, List) and not isinstance(value, Dict) and not isinstance(value, String):
+            return res.failure(IndexingError(
+                node.pos_start, node.pos_end,
+                "Can only get idx of list, dict, or string",
+                context
+            ))
+
         for idx_to_get in idxes_to_get:
             idx_to_get = res.register(self.visit(idx_to_get, context))
             idx_to_get = idx_to_get.value
@@ -129,7 +139,7 @@ class Interpreter:
                 if not isinstance(idx_to_get, int):
                     return res.failure(IndexingError(
                         node.pos_start, node.pos_end,
-                        f'List index must be an int',
+                        'List index must be an int',
                         context
                     ))
                 # If first statement is false, second will not evaluate
@@ -162,11 +172,10 @@ class Interpreter:
                         context
                     ))
             elif isinstance(value, String):
-                print(hasattr(value, "elements"))
                 if not isinstance(idx_to_get, int):
                     return res.failure(IndexingError(
                         node.pos_start, node.pos_end,
-                        f'String index must be an int',
+                        'String index must be an int',
                         context
                     ))
                 if idx_to_get < len(value.value):
@@ -180,40 +189,38 @@ class Interpreter:
             else:
                 return res.failure(TypingError(
                     node.pos_start, node.pos_end,
-                    'Can only get idx of list, dict, or string',
+                    'Index out of bounds',
                     context
                 ))
-        if attr_to_get:
-            if hasattr(value, "elements"):
-                value = value.elements
 
+        for attr_to_get in attrs_to_get:
             value_attr = value.attributes.get(attr_to_get, None)
-            if value_attr == None:
+            if value_attr is None:
                 return res.failure(AttrError(
                     node.pos_start, node.pos_end,
-                    f"'{var_name}' does not have the attribute '{attr_to_get}'",
+                    f"'{var_name_to_get}' does not have the attribute '{attr_to_get}'",
                     context
                 ))
 
-            value_attr = Number(value_attr)
-            return res.success(value_attr)
-        # idxes to get will have elements but value will no elements if value is a dict
-        return res.success(value.elements if isinstance(value, List) else value)
+            value = value_attr
+        return res.success(value)
 
     def visit_VarAssignNode(self, node, context):
         res = RTResult()
-        idxes_to_change = node.idxes_to_change
         assign_type = node.assign_type
 
-        if len(node.var_names_to_set) != len(node.values_to_set):
+        if len(node.vars_to_set) != len(node.values_to_set):
             return res.failure(TypingError(
                 node.pos_start, node.pos_end,
                 'Must have the same number of variables and values',
                 context
             ))
 
-        for var_name, value in zip(node.var_names_to_set, node.values_to_set):
+        for var, value in zip(node.vars_to_set, node.values_to_set):
             value = res.register(self.visit(value, context))
+            var_name = var[0]
+            idxes_to_change = var[1]
+
             if res.should_return():
                 return res
 
@@ -227,32 +234,31 @@ class Interpreter:
 
                 var_to_change = context.symbol_table.get(var_name)
 
-                if isinstance(var_to_change, List):
-                    element_to_change = var_to_change
+                element_to_change = var_to_change
 
-                    for for_idx, idx_to_change in enumerate(idxes_to_change):
-                        if isinstance(element_to_change, List):
-                            if idx_to_change.tok.value > len(element_to_change.elements) - 1:
-                                return res.failure(IndexingError(
-                                    node.pos_start, node.pos_end,
-                                    'List index out of bounds',
-                                    context
-                                ))
-                            if for_idx == len(idxes_to_change) - 1:
-                                element_to_change.elements[idx_to_change.tok.value] = value
-                            else:
-                                element_to_change = element_to_change.elements[idx_to_change.tok.value]
-                        elif isinstance(element_to_change, Dict) or isinstance(element_to_change.elements, Dict):
-                            if for_idx == len(idxes_to_change) - 1:
-                                element_to_change.key_pairs[idx_to_change.tok.value] = value
-                            else:
-                                element_to_change = element_to_change.key_pairs[idx_to_change.tok.value]
-                        else:
+                for for_idx, idx_to_change in enumerate(idxes_to_change):
+                    if isinstance(element_to_change, List):
+                        if idx_to_change.value > len(element_to_change.elements) - 1:
                             return res.failure(IndexingError(
                                 node.pos_start, node.pos_end,
-                                'Variable must be a list or dict in order to set a specific index',
+                                'List index out of bounds',
                                 context
                             ))
+                        if for_idx == len(idxes_to_change) - 1:
+                            element_to_change.elements[idx_to_change.value] = value
+                        else:
+                            element_to_change = element_to_change.elements[idx_to_change.value]
+                    elif isinstance(element_to_change, Dict) or (hasattr(element_to_change, "elements") and isinstance(element_to_change.elements, Dict)):
+                        if for_idx == len(idxes_to_change) - 1:
+                            element_to_change.key_pairs[idx_to_change.value] = value
+                        else:
+                            element_to_change = element_to_change.key_pairs[idx_to_change.value]
+                    else:
+                        return res.failure(IndexingError(
+                            node.pos_start, node.pos_end,
+                            f'Variable must be a list or dict not \'{type(element_to_change).__name__}\' in order to set a specific index',
+                            context
+                        ))
 
                 value = var_to_change
 
@@ -260,7 +266,7 @@ class Interpreter:
                 error = None
                 old_value = context.symbol_table.get(var_name)
 
-                if old_value == None:
+                if old_value is None:
                     return res.failure(NamingError(
                         node.pos_start, node.pos_end,
                         f"'{var_name}' is not defined",
@@ -292,31 +298,31 @@ class Interpreter:
         if res.should_return():
             return res
 
-        if node.op_tok.type == TT_PLUS:
+        if node.op_tok.type == toks.TT_PLUS:
             result, error = left.added_to(right)
-        elif node.op_tok.type == TT_MINUS:
+        elif node.op_tok.type == toks.TT_MINUS:
             result, error = left.subbed_by(right)
-        elif node.op_tok.type == TT_MUL:
+        elif node.op_tok.type == toks.TT_MUL:
             result, error = left.multed_by(right)
-        elif node.op_tok.type == TT_DIV:
+        elif node.op_tok.type == toks.TT_DIV:
             result, error = left.dived_by(right)
-        elif node.op_tok.type == TT_POW:
+        elif node.op_tok.type == toks.TT_POW:
             result, error = left.powed_by(right)
-        elif node.op_tok.type == TT_EE:
+        elif node.op_tok.type == toks.TT_EE:
             result, error = left.get_comparison_eq(right)
-        elif node.op_tok.type == TT_NE:
+        elif node.op_tok.type == toks.TT_NE:
             result, error = left.get_comparison_ne(right)
-        elif node.op_tok.type == TT_LT:
+        elif node.op_tok.type == toks.TT_LT:
             result, error = left.get_comparison_lt(right)
-        elif node.op_tok.type == TT_GT:
+        elif node.op_tok.type == toks.TT_GT:
             result, error = left.get_comparison_gt(right)
-        elif node.op_tok.type == TT_LTE:
+        elif node.op_tok.type == toks.TT_LTE:
             result, error = left.get_comparison_lte(right)
-        elif node.op_tok.type == TT_GTE:
+        elif node.op_tok.type == toks.TT_GTE:
             result, error = left.get_comparison_gte(right)
-        elif node.op_tok.matches(TT_KEYWORD, 'et'):
+        elif node.op_tok.matches(toks.TT_KEYWORD, 'et'):
             result, error = left.anded_by(right)
-        elif node.op_tok.matches(TT_KEYWORD, 'aut'):
+        elif node.op_tok.matches(toks.TT_KEYWORD, 'aut'):
             result, error = left.ored_by(right)
 
         if error:
@@ -332,9 +338,9 @@ class Interpreter:
 
         error = None
 
-        if node.op_tok.type == TT_MINUS:
+        if node.op_tok.type == toks.TT_MINUS:
             number, error = number.multed_by(Number(-1))
-        elif node.op_tok.matches(TT_KEYWORD, 'non'):
+        elif node.op_tok.matches(toks.TT_KEYWORD, 'non'):
             number, error = number.notted()
 
         if error:
@@ -350,7 +356,7 @@ class Interpreter:
             if res.should_return():
                 return res
 
-            if condition_value == None:
+            if condition_value is None:
                 return res.failure(RTError(
                     node.pos_start, node.pos_end,
                     'Conditional can not be evaluated',
@@ -383,7 +389,8 @@ class Interpreter:
                 context
             ))
 
-        if try_body.error and node.except_body and (not node.except_name or node.except_name.value == "Exception" or try_body.error.error_name == node.except_name.value):
+        should_except = not node.except_name or node.except_name.value == "Exception" or try_body.error.error_name == node.except_name.value
+        if try_body.error and node.except_body and should_except:
             if node.except_as:
                 context.symbol_table.set(
                     node.except_as.value, String(try_body.error.details))
@@ -394,11 +401,12 @@ class Interpreter:
 
             if node.except_as:
                 context.symbol_table.remove(node.except_as.value)
-        elif try_body.error == None:
+        elif try_body.error is None:
             try_body = res.register(try_body)
             if res.error:
                 return res
-        return res.success(None)
+
+        return res.success(None if node.should_return_null else try_body)
 
     def visit_ForNode(self, node, context):
         res = RTResult()
@@ -420,7 +428,7 @@ class Interpreter:
         else:
             step_value = Number(1)
 
-        if start_value == None:
+        if start_value is None:
             return res.failure(RTError(
                 node.start_value_node.pos_start, node.start_value_node.pos_end,
                 'Expression does not have a value',
@@ -430,16 +438,18 @@ class Interpreter:
         i = start_value.value
 
         if step_value.value >= 0:
-            def condition(): return i < end_value.value
+            def condition():
+                return i < end_value.value
         else:
-            def condition(): return i > end_value.value
+            def condition():
+                return i > end_value.value
 
         while condition():
-            context.symbol_table.set(node.var_name_tok.value, Number(i))
+            context.symbol_table.set(node.var_name, Number(i))
             i += step_value.value
 
             value = res.register(self.visit(node.body_node, context))
-            if res.should_return() and res.loop_should_continue == False and res.loop_should_break == False:
+            if res.should_return() and res.loop_should_continue and res.loop_should_break:
                 return res
 
             if res.loop_should_continue:
@@ -450,7 +460,7 @@ class Interpreter:
 
             elements.append(value)
 
-        context.symbol_table.remove(node.var_name_tok.value)
+        context.symbol_table.remove(node.var_name)
         return res.success(
             None if node.should_return_null else
             List(elements).set_context(context).set_pos(
@@ -467,7 +477,7 @@ class Interpreter:
             if res.should_return():
                 return res
 
-            if condition_value == None:
+            if condition_value is None:
                 return res.failure(RTError(
                     node.condition_node.pos_start, node.condition_node.pos_end,
                     'Conditional can not be evaluated',
@@ -478,7 +488,7 @@ class Interpreter:
                 break
 
             value = res.register(self.visit(node.body_node, context))
-            if res.should_return() and res.loop_should_continue == False and res.loop_should_break == False:
+            if res.should_return() and res.loop_should_continue and res.loop_should_break:
                 return res
 
             if res.loop_should_continue:
@@ -498,35 +508,54 @@ class Interpreter:
     def visit_FuncDefNode(self, node, context):
         res = RTResult()
 
-        func_name = node.var_name_tok.value if node.var_name_tok else None
+        func_name = node.func_name if node.func_name else None
         body_node = node.body_node
-        arg_names = [arg_name.value for arg_name in node.arg_name_toks]
-        func_value = Function(func_name, body_node, arg_names, node.should_auto_return).set_context(
+        args = []
+        for arg_name, arg_value in node.args:
+            if arg_value is None:
+                args.append((arg_name, arg_value))
+            else:
+                arg_value = res.register(self.visit(arg_value, context))
+                if res.should_return():
+                    return res
+                args.append((arg_name, arg_value))
+
+        func_value = Function(func_name, body_node, args, node.should_auto_return).set_context(
             context).set_pos(node.pos_start, node.pos_end)
 
-        if node.var_name_tok:
+        if node.func_name:
             context.symbol_table.set(func_name, func_value)
 
         return res.success(func_value)
 
     def visit_CallNode(self, node, context):
+
+        if self.recursion_depth > 100:
+            return RTResult().failure(RecursingError(
+                node.pos_start, node.pos_end,
+                'Recursion depth exceeded',
+                context
+            ))
+
         res = RTResult()
-        args = []
+        args = {}
 
         value_to_call = res.register(self.visit(node.node_to_call, context))
         if res.should_return():
             return res
         value_to_call = value_to_call.copy().set_pos(node.pos_start, node.pos_end)
 
-        for arg_node in node.arg_nodes:
-            args.append(res.register(self.visit(arg_node, context)))
+        for arg_key, arg_value in node.arg_nodes.items():
+            arg_value = res.register(self.visit(arg_value, context))
             if res.should_return():
                 return res
 
-        return_value = res.register(value_to_call.execute(args))
+            args[arg_key] = arg_value
+
+        return_value = res.register(value_to_call.execute(args, self.recursion_depth))
         if res.should_return():
             return res
-        if return_value == None:
+        if return_value is None:
             return res.success(None)
 
         return_value = return_value.copy().set_pos(
