@@ -1,7 +1,15 @@
-from .BaseFunction import BaseFunction
+#####################################
+# IMPORTS
+#####################################
+
 from Celeratas.interpreter.RTResult import RTResult
 
+from .BaseFunction import BaseFunction
 
+
+#####################################
+# FUNCTION
+#####################################
 class Function(BaseFunction):
     def __init__(self, name, body_node, arg_names, should_auto_return):
         super().__init__(name)
@@ -9,10 +17,10 @@ class Function(BaseFunction):
         self.arg_names = arg_names
         self.should_auto_return = should_auto_return
 
-    def execute(self, args):
+    def execute(self, args, recursion_depth):
         res = RTResult()
-        from interpreter.Interpreter import Interpreter
-        interpreter = Interpreter()
+        from Celeratas.interpreter.Interpreter import Interpreter
+        interpreter = Interpreter(recursion_depth + 1)
         exec_ctx = self.generate_new_context()
 
         res.register(self.check_and_populate_args(
@@ -21,7 +29,7 @@ class Function(BaseFunction):
             return res
 
         value = res.register(interpreter.visit(self.body_node, exec_ctx))
-        if res.should_return() and res.func_return_value == None:
+        if res.should_return() and res.func_return_value is None:
             return res
 
         ret_value = (
